#include "GLSurface.h"

GLSurface::GLSurface() {
	lastTicks = SDL_GetTicks();
	animTime = 0;
	tex = 0;
	alpha = 1.0f;
	animating = false;
}

GLSurface::GLSurface(std::string texture) {
	tex = textureManager.loadTexture(texture);
	lastTicks = SDL_GetTicks();
	animTime = 0;
	alpha = 1.0f;
	animating = false;
}

GLSurface::~GLSurface(void)
{
}

void GLSurface::parseAnim(void) {
	if (!animating || qEffects.empty()) return;

	SurfAnimation curAnim = qEffects.front();
	unsigned int curTime = SDL_GetTicks();

	animTime += curTime - lastTicks;

	switch (curAnim.effect) {
	case FADE_IN: alpha = lerpf(0.0f, 1.0f, ((float)animTime / (float)curAnim.duration)); break;
	case FADE_OUT: alpha = lerpf(1.0f, 0.0f, ((float)animTime / (float)curAnim.duration)); break;
	default: break;
	}

	if (animTime > curAnim.duration) {
		switch (curAnim.effect) {
		case FADE_IN: alpha = 1.0f; break;
		case FADE_OUT: alpha = 0.0f; break;
		default: break;
		}
		qEffects.pop_front();
		if (qEffects.empty()) animating = false;
		animTime = 0;
	}

	lastTicks = curTime;

}

void GLSurface::renderEntity(void) {
	glDisable(GL_LIGHTING);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glEnable(GL_TEXTURE_2D);

	parseAnim();

	glBegin(GL_QUADS);
		glColor4f(1.0f, 1.0, 1.0, alpha);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(-0.5f, -0.5f);
			
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(-0.5f, 0.5f);
			
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(0.5f, 0.5f);

		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(0.5f, -0.5f);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}

void GLSurface::addAnimation(SurfEffect e, unsigned int ms) {
	if (!animating) {
		animTime = 0;
		lastTicks = SDL_GetTicks();
	}
	SurfAnimation a = {e, ms};
	qEffects.push_back(a);
	animating = true;
}

void GLSurface::stopAnimation(void) {
	qEffects.clear();
	animating = false;
}

void GLSurface::fadeIn(unsigned int ms) {
	addAnimation(FADE_IN, ms);
}

void GLSurface::fadeOut(unsigned int ms) {
	addAnimation(FADE_OUT, ms);
}

void GLSurface::idle(unsigned int ms) {
	addAnimation(IDLE, ms);
}