#include "HUD.h"

extern TextureManager textureManager;
extern Config cfg;

HUD::HUD(void) {
	surfPointer = GLSurface("pointer.png");
	surfPointerBack = GLSurface("pointer_back.png");
	surfGameOver = GLSurface("gameover.png");
	surfWin = GLSurface("win.png");
	surfHP = GLSurface("hp_bar.png");
	surfHPBack = GLSurface("hp_bar_back.png");
	surfCheckpoint = GLSurface("checkpoint.png");

	const float diameter = 0.3f;
	surfPointerBack.sx = surfPointer.sx = diameter;
	surfPointerBack.sy = surfPointer.sy = diameter;

	surfPointerBack.x = surfPointer.x = 0.0f;
	surfPointerBack.y = surfPointer.y = 0.3f;

	surfHPBack.sy = surfHP.sy = 0.05f;
	surfHPBack.y = surfHP.y = 0.48f;

	surfCheckpoint.sx = 0.5f;
	surfCheckpoint.sy = 0.1f;
	surfCheckpoint.y = -0.3f;
	surfCheckpoint.alpha = 0.0f;
}

HUD::~HUD(void) {
}

void HUD::renderSpeed(float speed) {
	const float minAngle = 225.0f;
	const float maxAngle = 495.0f;

	float angle = (((minAngle) + (speed * (maxAngle - minAngle))));// * PIOVER180;
	
//	surfPointer.rz = angle;
	surfPointer.roll = angle;

	glPushMatrix();
	surfPointerBack.render();
	glPopMatrix();
	
	glPushMatrix();
	surfPointer.render();
	glPopMatrix();
}

void HUD::render(HUDData& data) {
	
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();

	if (cfg.screenW > cfg.screenH) {
		glViewport((cfg.screenW - cfg.screenH) / 2, 0, cfg.screenH, cfg.screenH);
	} else {
		glViewport(0, (cfg.screenH - cfg.screenW) / 2, cfg.screenW, cfg.screenW);
	}
    gluOrtho2D(-0.5, 0.5, -0.5, 0.5);
    glScalef(1.0f, -1.0f, 1.0f);
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	renderSpeed(data.speed);
	glPopMatrix();
	
	glPushMatrix();
	glLoadIdentity();
	surfHPBack.render();
	glPopMatrix();

	if (data.checkpoint) {
		surfCheckpoint.stopAnimation();
		surfCheckpoint.fadeIn(1000);
		surfCheckpoint.idle(3000);
		surfCheckpoint.fadeOut(3000);
		data.checkpoint = false;
	}
	glPushMatrix();
	glLoadIdentity();
	surfCheckpoint.render();
	glPopMatrix();

	glPushMatrix();
	glLoadIdentity();
	surfHP.sx = data.hp;
	surfHP.render();
	glPopMatrix();

	if (data.state == gameover) { 
		glViewport(0, 0, cfg.screenW, cfg.screenH);
		glLoadIdentity();
		glPushMatrix();
		surfGameOver.render();
		glPopMatrix();
	} else if (data.state == win) {
		glViewport(0, 0, cfg.screenW, cfg.screenH);
		glLoadIdentity();
		glPushMatrix();
		surfWin.render();
		glPopMatrix();
	}

}
