#include "Model.h"

Model::Model(void) {
#ifdef USE_VBOS
	nVBOs = 0;
	indicesVBOID = 0;
#endif
}

Model::~Model(void) {
#ifdef USE_VBOS
	for (int i = 0; i < nVBOs; i++) {
		glDeleteBuffers(1, &VBOIDs[i]);
	}
	if (nVBOs > 0) delete[] VBOIDs;
	if (indicesVBOID) glDeleteBuffers(1, &indicesVBOID);
#endif
}

void applyMaterial(Material mat) {
	GLfloat mat_ambient[] = { mat.ar, mat.ag, mat.ab, mat.alpha };
	GLfloat mat_specular[] = { mat.sr, mat.sg, mat.sb, mat.alpha };
	GLfloat mat_diffuse[] = { mat.dr, mat.dg, mat.db, mat.alpha };
	GLfloat mat_shininess[] = { mat.shininess };

	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

	if (mat.textureVBOID > 0) {
		glBindTexture(GL_TEXTURE_2D, mat.textureVBOID);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

		glEnable(GL_TEXTURE_2D);
	} else {
		glDisable(GL_TEXTURE_2D);
	}
}

void Model::render(void) {
	// FIXME: quick material hack
	applyMaterial(materials[0]);
	for (int i = 0; i < nVBOs; i++) {
		glBindBuffer(GL_ARRAY_BUFFER, VBOIDs[i]);
	
		glVertexPointer(3, GL_FLOAT, sizeof(Vertex), (GLfloat*)0+0);
		glNormalPointer(GL_FLOAT, sizeof(Vertex),
			BUFFER_OFFSET(sizeof(GLfloat) * 3));

		glClientActiveTexture(GL_TEXTURE0);
		glTexCoordPointer(2, GL_FLOAT, sizeof(Vertex), BUFFER_OFFSET(sizeof(GLfloat) * 6));

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indicesVBOID);

		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, (GLuint*)0+0);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	}
}

void Model::buildVBOs() {
	GLsizeiptr vbosize;
	nVBOs = 1;
	VBOIDs = new GLuint[nVBOs];

	vbosize = sizeof(Vertex) * nVertices;
	glGenBuffers(1, &VBOIDs[0]);
	glBindBuffer(GL_ARRAY_BUFFER, VBOIDs[0]);
	glBufferData(GL_ARRAY_BUFFER, vbosize, vertices, GL_STATIC_DRAW);
	//delete[] vertices;

	// Create a index buffer
	vbosize = sizeof(GLuint) * nIndices;
	glGenBuffers(1, &indicesVBOID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indicesVBOID);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, vbosize, indices, GL_STATIC_DRAW);
	//delete[] indices;
}
