#include <stdlib.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include "utils.h"

float randnum (float min, float max)
{
	int r;
    float	x;
    
	r = rand ();
    x = (float)(r & 0x7fff) /
		(float)0x7fff;
    return (x * (max - min) + min);
} 

void vectorDiff(const int d, const float* vec1, const float* vec2, float* ret) {
	for (int i = 0; i < d; i++) {
		ret[i] = vec1[i] - vec2[i];
	}
}

void normalize(const int d, float* vec) {
	float len = 0.0f;
	for (int i = 0; i < d; i++) {
		len += vec[i] * vec[i];
	}
	len = sqrtf(len);

	for (int i = 0; i < d; i++) {
		vec[i] /= len;
	}
}

void calcNormal(const float triangle[3][3], float normal[3]) {
	float u[3], v[3];
	
	vectorDiff(3, triangle[1], triangle[0], u);
	vectorDiff(3, triangle[2], triangle[1], v);

	normal[0] = -((u[1] * v[2]) - (u[2] * v[1]));
	normal[1] = -((u[0] * v[2]) - (u[2] * v[0]));
	normal[2] = -((u[0] * v[1]) - (u[1] * v[0]));

	normalize(3, normal);
}

GLuint createShader(GLenum shaderType, const std::string& filename) {
	GLuint shader = glCreateShader(shaderType);
	std::string fullFileName = SHADER_PATH + filename;
	const char* strFileName = fullFileName.c_str();
	glShaderSource(shader, 1, &strFileName, NULL);

	glCompileShader(shader);

	GLint status;
	glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
	if (status == GL_FALSE) {
		GLint infoLogLen;
		glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLogLen);

		GLchar* infoLog = new GLchar[infoLogLen + 1];
		glGetShaderInfoLog(shader, infoLogLen, NULL, infoLog);

		const char* strShaderType = NULL;
		switch (shaderType) {
		case GL_VERTEX_SHADER: strShaderType = "vertex"; break;
		case GL_GEOMETRY_SHADER: strShaderType = "geometry"; break;
		case GL_FRAGMENT_SHADER: strShaderType = "fragment"; break;
		}

		fprintf(stderr, "<%s>: %s\n", strShaderType, infoLog);
		delete[] infoLog;
	}

	return shader;
}

float lerpf(float a, float b, float amt) {
	float diff = b - a;
	return a + (diff * amt);
}

Json::Value readJSONDoc(std::string file, Json::Reader& reader) {
	std::ifstream stream;
	Json::Value root;
	stream.open(file.c_str(), std::ifstream::in);

	bool success = reader.parse(stream, root, false);
	stream.close();

	return root;
}


